
/* ShapeType_publisher.cxx

A publication of data of type rti::shapes::ShapeType

This file is derived from code automatically generated by the rtiddsgen 
command:

rtiddsgen -language C++ -example <arch> ShapeType.idl

Example publication of type rti::shapes::ShapeType automatically generated by
'rtiddsgen'. To test them follow these steps:

(1) Compile this file and the example subscription.

(2) Start the subscription with the command
objs/<arch>/ShapeType_subscriber <domain_id> <sample_count>

(3) Start the publication with the command
objs/<arch>/ShapeType_publisher <domain_id> <sample_count>

(4) [Optional] Specify the list of discovery initial peers and 
multicast receive addresses via an environment variable or a file 
(in the current working directory) called NDDS_DISCOVERY_PEERS. 

You can run any number of publishers and subscribers programs, and can 
add and remove them dynamically from the domain.

Example:

To run the example application on domain <domain_id>:

On Unix: 

objs/<arch>/ShapeType_publisher <domain_id> o
objs/<arch>/ShapeType_subscriber <domain_id> 

On Windows:

objs\<arch>\ShapeType_publisher <domain_id>  
objs\<arch>\ShapeType_subscriber <domain_id>    

*/

#include <stdio.h>
#include <stdlib.h>
#include "ShapeType.h"
#include "ShapeTypeSupport.h"
#include "ndds/ndds_cpp.h"

#include "ndds/ndds_namespace_cpp.h"

using namespace DDS;

/* Delete all entities */
static int publisher_shutdown(
    DomainParticipant *participant)
{
    ReturnCode_t retcode;
    int status = 0;

    if (participant != NULL) {
        retcode = participant->delete_contained_entities();
        if (retcode != RETCODE_OK) {
            printf("delete_contained_entities error %d\n", retcode);
            status = -1;
        }

        retcode = TheParticipantFactory->delete_participant(participant);
        if (retcode != RETCODE_OK) {
            printf("delete_participant error %d\n", retcode);
            status = -1;
        }
    }

    /* RTI Connext provides finalize_instance() method on
    domain participant factory for people who want to release memory used
    by the participant factory. Uncomment the following block of code for
    clean destruction of the singleton. */
    /*
    retcode = DomainParticipantFactory::finalize_instance();
    if (retcode != RETCODE_OK) {
        printf("finalize_instance error %d\n", retcode);
        status = -1;
    }
    */

    return status;
}

extern "C" int publisher_main(int domainId, int sample_count)
{
    DomainParticipant *participant = NULL;
    rti::shapes::ShapeTypeDataWriter * ShapeType_writer = NULL;
    ReturnCode_t retcode;
    const char *type_name = NULL;
    int count = 0;  
    Duration_t send_period = {4,0};

    /* Register types */
    retcode = TheParticipantFactory->register_type_support(
    		rti::shapes::ShapeTypeTypeSupport::register_type,
    		                        My::Topic::Shape::TYPE.c_str());
    if (retcode != RETCODE_OK) {
        printf("register_type error %d\n", retcode);
        publisher_shutdown(NULL);
        return -1;
    }

    /* Create Participant from XML Config */
    participant = TheParticipantFactory->
            create_participant_from_config(My::If::PUB.c_str());
    if (participant == NULL) {
        printf("create_participant_from_config error\n");
        publisher_shutdown(participant);
        return -1;
    }

    /* Lookup the DataWriter */
    ShapeType_writer = rti::shapes::ShapeTypeDataWriter::narrow(
    		participant->lookup_datawriter_by_name(
    		                    My::Topic::Shape::WRITER.c_str()));
    if (ShapeType_writer == NULL) {
        printf("DataWriter narrow error\n");
        publisher_shutdown(participant);
        return -1;
    }


    /* Create data sample for writing */
    rti::shapes::ShapeType sample;


    /* For a data type that has a key, if the same instance is going to be
    written multiple times, initialize the key here
    and register the keyed instance prior to writing */
    sample.color= "RED";
    InstanceHandle_t instance_handle = HANDLE_NIL;
    //instance_handle = ShapeType_writer->register_instance(sample);


    /* Main loop */
    for (count=0; (sample_count == 0) || (count < sample_count); ++count) {

        printf("Writing rti::shapes::ShapeType, count %d\n", count);

        /* Modify the data to be sent here */
        sample.x = 20 + (count*10) % 230;
        sample.y = 100 + (count*10) % 230;
        sample.shapesize = 40;

        retcode = ShapeType_writer->write(sample, instance_handle);
        if (retcode != RETCODE_OK) {
            printf("write error %d\n", retcode);
        }

        NDDSUtility::sleep(send_period);
    }

    /*
    retcode = ShapeType_writer->unregister_instance(
        *instance, instance_handle);
    if (retcode != RETCODE_OK) {
        printf("unregister instance error %d\n", retcode);
    }
    */

    /* Delete all entities */
    return publisher_shutdown(participant);
}

int main(int argc, char *argv[])
{
    int domainId = 0;
    int sample_count = 0; /* infinite loop */

    if (argc >= 2) {
        domainId = atoi(argv[1]);
    }
    if (argc >= 3) {
        sample_count = atoi(argv[2]);
    }

    /* Uncomment this to turn on additional logging
    NDDSConfigLogger::get_instance()->
    set_verbosity_by_category(NDDS_CONFIG_LOG_CATEGORY_API, 
    NDDS_CONFIG_LOG_VERBOSITY_STATUS_ALL);
    */

    return publisher_main(domainId, sample_count);
}

